//#include <Arduino.h>
#include <DFRobotHCHOSensor.h>//记得在IDE加载HCHO传感器库文件
#include <SoftwareSerial.h>
#include <LiquidCrystal_I2C.h>//修改成自己购买的1602模块库文件名称（记得事先在IDE加载库文件）
#include <Wire.h>
#define SensorSerialPin  9  //this pin read the uart signal from the HCHO sensor
#define WIFISSID "DFRobot-guest" //修改成自己的WIFI账号
#define WIFIPWD  "dfrobot@2017" //修改成自己的WIFI密码
#define SERVER   "iot.dfrobot.com.cn"
#define PORT     1883
#define IOTID    "xxxxxxxxxx"//修改成自己的IOTID
#define IOTPWD   "xxxxxxxxxx"//修改成自己的IOTPWD
LiquidCrystal_I2C lcd(0x27,16,2);
SoftwareSerial softSerial(10,11);//定义IOT软串口
SoftwareSerial sensorSerial(SensorSerialPin,SensorSerialPin);//定义HCHO传感器软串口
DFRobotHCHOSensor hchoSensor(&sensorSerial);
 
const String separator = "|";
bool pingOn = true;
bool obloqConnectMqtt = false;
static unsigned long pingInterval = 2000;
static unsigned long sendMessageInterval = 10000;
unsigned long previousPingTime = 0;
unsigned long previousSendMessageTime = 0;
 
//wifi异常断开检测变量
bool wifiConnect = false;
bool wifiAbnormalDisconnect = false;
 
//mqtt因为网络断开后重新连接标志
bool mqttReconnectFlag = false;
enum state{
    WAIT,
    PINGOK,
    WIFIOK,
    MQTTCONNECTOK
}obloqState;
 
/********************************************************************************************
Function    : sendMessage      
Description : 通过串口向OBLOQ发送一次消息
Params      : 无
Return      : 无 
********************************************************************************************/
void sendMessage(String message)
{
    softSerial.print(message+"\r"); 
}
 
/********************************************************************************************
Function    : ping      
Description : 通过串口向OBLOQ发送一次字符串"|1|1|"，尝试与OBLOQ取得连接
Params      : 无
Return      : 无 
********************************************************************************************/
void ping()
{
    String pingMessage = "|1|1|";
    sendMessage(pingMessage);
}
 
/********************************************************************************************
Function    : connectWifi      
Description : 连接wifi   
Params      : ssid 连接wifi的ssid；pwd 连接wifi的password
Return      : 无 
********************************************************************************************/
void connectWifi(String ssid,String pwd)
{
    String wifiMessage = "|2|1|" + ssid + "," + pwd + separator;
    sendMessage(wifiMessage);
} 
 
/********************************************************************************************
Function    : connectMqtt      
Description : 连接DF-IoT    
Params      : server 物联网网址；port 端口；iotid 物联网登录后分配的iotid；iotpwd 物联网登录后分配的iotpwd
Return      : 无 
********************************************************************************************/
void connectMqtt(String server, String port, String iotid, String iotpwd)
{
    String mqttConnectMessage = "|4|1|1|" + server + separator + port + separator + iotid + separator + iotpwd + separator;
    sendMessage(mqttConnectMessage);
}
 
 
/********************************************************************************************
Function    : reconnectMqtt      
Description : 重新连接DF-IoT    
Params      : 无 
Return      : 无 
********************************************************************************************/
void reconnectMqtt()
{
    String mqttReconnectMessage = "|4|1|5|"; 
    sendMessage(mqttReconnectMessage);
}
 
 
/********************************************************************************************
Function    : publish      
Description : 向DF-IoT物联网设备发送信息    
Params      : topic DF-IoT物联网设备编号；message 发送的消息内容
Return      : 无 
********************************************************************************************/
void publish(String topic,String message)
{
    String publishMessage = "|4|1|3|" + topic + separator + message + separator;
    sendMessage(publishMessage);
}
 
/********************************************************************************************
Function    : handleUart      
Description : 处理串口传回的数据      
Params      : 无   
Return      : 无 
********************************************************************************************/
void handleUart()
{
    while(softSerial.available() > 0)
    {
        String receivedata = softSerial.readStringUntil('\r');
        const char* obloqMessage = receivedata.c_str();
        if (strcmp(obloqMessage, "|1|1|") == 0)
    {
      Serial.println("Pong");
      pingOn = false;
      obloqState = PINGOK;
    }
        if(strcmp(obloqMessage, "|2|1|") == 0)
        {
            if(wifiConnect)
            {
                wifiConnect = false;
                wifiAbnormalDisconnect = true;
            }
        }
    else if (strstr(obloqMessage,"|2|3|") != NULL && strlen(obloqMessage) != 9)
    {
      Serial.println("Wifi ready");
            wifiConnect = true;
            if(wifiAbnormalDisconnect)
            {
                wifiAbnormalDisconnect = false;
                return;
            }
      obloqState = WIFIOK;
    }
    else if (strcmp(obloqMessage, "|4|1|1|1|") == 0)
    {
      Serial.println("Mqtt ready");
            obloqConnectMqtt = true;
            if(mqttReconnectFlag)
            {
                mqttReconnectFlag = false;
                return;
            }
            obloqState = MQTTCONNECTOK;
    }
    }
}
 
/********************************************************************************************
Function    : sendPing      
Description : 每隔pingInterval一段时间，通过串口向OBLOQ ping一次      
Params      : 无   
Return      : 无 
********************************************************************************************/
void sendPing()
{
    if(pingOn && millis() - previousPingTime > pingInterval)
    {
        previousPingTime = millis();
        ping();
    }
}
 
/********************************************************************************************
Function    : execute      
Description : 根据OBLOQ的状态，进行下一步相应操作。      
Params      : 无
Return      : 无 
********************************************************************************************/
void execute()
{
    switch(obloqState)
    {
        case PINGOK: connectWifi(WIFISSID,WIFIPWD); obloqState = WAIT; break;
        case WIFIOK: connectMqtt(SERVER,String(PORT),IOTID,IOTPWD);obloqState = WAIT; break;
        case MQTTCONNECTOK : obloqState = WAIT; break;
        default: break;
    }
}
 
/********************************************************************************************
Function    : checkWifiState      
Description : 检查wifi状态,如果检测到wifi热点断开会间隔1分钟去重新连接wifi
Params      : 无
Return      : 无
********************************************************************************************/
void checkWifiState()
{
    static unsigned long previousTime = 0;
    static bool reconnectWifi = false;
    if(wifiAbnormalDisconnect && millis() - previousTime > 60000)
    {
        previousTime = millis();
        Serial.println("Wifi abnormal disconnect");
        reconnectWifi = true;
        obloqConnectMqtt = false;
        connectWifi(WIFISSID,WIFIPWD);
    }
    if(!wifiAbnormalDisconnect && reconnectWifi)
    {
        reconnectWifi = false;
        mqttReconnectFlag = true;
        Serial.println("Reconnect mqtt");
        reconnectMqtt();
    }
 
}
 
/********************************************************************************************
Function    : getTemp      
Description : 获得LM35测得的温度
Params      : 无
Return      : 无 
********************************************************************************************/
float getHCHO()
{
    uint16_t val;
    float dat;
    dat = hchoSensor.uartReadPPM()*1.34;
    return dat;
}
 
void setup()
{  
    sensorSerial.begin(9600); //the baudrate of HCHO is 9600
    sensorSerial.listen();
    Serial.begin(9600);
    lcd.init();
    lcd.backlight(); 
    lcd.setCursor(1, 0 );
    lcd.print("HCHO:");
    lcd.setCursor(1, 1 );
}
 
void loop()
{
    //sensorSerial.listen();//监听HCHO传感器软串口
    if(hchoSensor.available()>0)  
    {
      //Serial.print(hchoSensor.uartReadPPM());
      //Serial.println("ppm");
     lcd.setCursor(6,1); 
     lcd.print(hchoSensor.uartReadPPM());
     lcd.print("ppm");
     lcd.setCursor(6,0);
     lcd.print(hchoSensor.uartReadPPM()*1.34);  //Unit conversion ：1ppm = 1.34 mg/m3 
     lcd.print("mg/m3");
        
    }
     // softSerial.listen();//监听IOT软串口
     //IOT部分
    sendPing();
    execute();
    handleUart();
    checkWifiState();
    if(obloqConnectMqtt && millis() - previousSendMessageTime > 5000) //每隔5s，设备发送消息，消息内容是此时读取的HCHO数据
    {
        previousSendMessageTime = millis();
        float HCHO=getHCHO();//获取甲醛数据
        publish("BXXXXXXX",(String)HCHO); //BXXXXXXX根据自己的IOT设备修改
        Serial.print(HCHO);
        Serial.println("ppm");   
        delay(100);
    }
       
}
